//###########################################################################
//
// FILE:	PulsesTest_DCAC-main.c
//
// TITLE:	Initialization and interrupt handling routing for pulses test
//
// This file contains the main routine where all the necessary peripherals
// are initialized, as well as the main Interrupt Service Routine (ISR)
// that manages the PWM outputs. It is possible to select whether this test
// is made on a full bridge or on a half-bridge (check allHeaderincludes.h),
// as well as the switching frequency, number of pulses, duty cycle, dead time.
// For the full bridge it is also possible to select the polarity (full bridge),
// and for the half bridge it is possible to select high-side switching or
// low-side switching.
//
// CODE TESTED USING CODE COMPOSER 6.2.0 and TMS320F28035 WITH XDS100V1

//###########################################################################
// Transphorm V1.0 (Ricardo Pregitzer)
// For questions please contact Transphorm Inc.
// Release Date: April 13, 2017
//###########################################################################

#include "allHeaderIncludes.h"
#include "DSP2803x_EPWM_defines.h"

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//FUNCTION PROTOTYPES
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

__interrupt void epwm1_timer_isr(void);
void DeviceInit(void);
void InitFlash();
void MemCopy();
extern void PWM_Config(int16 n, int16 Period);

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - GENERAL
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// Used for running Background in flash, and ISRs in RAM
extern Uint16 *RamfuncsLoadStart, *RamfuncsLoadEnd, *RamfuncsRunStart;

// Main variables for pulses test
Uint32  EPwm1TimerIntCount=0; //Control variable for the software to determine when to disable the switching
int16 Reset=0;		//This register will immediately trigger a new run when set to 1
int16 Polarity=0;	//Polarity will either change the switching leg (full bridge unipolar),
					//or which one is the discharge HEMT (for half bridge)
int16 Pulses=4;		//Total number of switching pulses in one run
int16 CMP=25;		//Duty Cycle
int16 Period=600; //Period Counter for the PWM (inversely proportional to the switching frequency)
Uint16 DeadTime=5; 	// Dead time is ~16.667ns per count @ 60MHz

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//MACROS DECLARATIONS for handling PWM outputs
#define		mP_PWM_OFF()		{EALLOW; EPwm1Regs.TZFRC.bit.OST = 1; EDIS;}
#define		mP_PWM_ON()			{EALLOW; EPwm1Regs.TZCLR.bit.OST = 1; EDIS;}
#define		mN_PWM_OFF()		{EALLOW; EPwm2Regs.TZFRC.bit.OST = 1; EDIS;}
#define		mN_PWM_ON()			{EALLOW; EPwm2Regs.TZCLR.bit.OST = 1; EDIS;}
#define		mALL_PWM_ON()		{EALLOW; \
								 EPwm1Regs.TZCLR.bit.OST = 1; \
								 EPwm2Regs.TZCLR.bit.OST = 1;\
								 EDIS;}
#define		mALL_PWM_OFF()		{EALLOW; \
								 EPwm1Regs.TZFRC.bit.OST = 1; \
								 EPwm2Regs.TZFRC.bit.OST = 1;\
								 EDIS;}
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// MAIN CODE
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void main(void)
{
//=================================================================================
//	INITIALISATION - General
//=================================================================================

	// The DeviceInit() configures the clocks and pin mux registers 
	// The function is declared in {ProjectName}-DevInit_F2803/2x.c,
	// Please ensure/edit that all the desired components pin muxes 
	// are configured properly that clocks for the peripherals used
	// are enabled, for example the individual PWM clock must be enabled 
	// along with the Time Base Clock 

	DeviceInit();	// Device Life support & GPIO

// Copy time critical code and Flash setup code to RAM
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the linker files. 
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

// Call Flash Initialization to setup flash wait states
// This function must reside in RAM
	InitFlash();	// Call the flash wrapper init function

//  Guarantee all PWM pins are off
	mALL_PWM_OFF();		//PWM disable

	// Configure PWM1~2
	PWM_Config(1, Period);
#if FULL_BRIDGE == 1
	PWM_Config(2, Period);

	// Sync PWM1 with PWM2 without phase shift
	EPwm2Regs.TBCTL.bit.PHSEN = TB_ENABLE;
	EPwm2Regs.TBCTL.bit.PHSDIR = TB_UP;
	EPwm2Regs.TBPHS.half.TBPHS = 0;
#endif

    EPwm1Regs.ETSEL.bit.INTSEL = 1;     // Select INT on Zero event
    EPwm1Regs.ETSEL.bit.INTEN = 1;  	// Enable INT
    EPwm1Regs.ETPS.bit.INTPRD = 1;      // Generate INT on 1st event


//==============================================================================
// Items common to all builds
//==============================================================================


//All enabled ePWM module clocks are started with the first rising edge of 
//TBCLK aligned   
	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
	EDIS;

//=================================================================================
//	INTERRUPT & ISR INITIALIZATION (best to run this section after other initializations)
//=================================================================================

	EALLOW;
	PieVectTable.EPWM1_INT = &epwm1_timer_isr;
	EDIS;

	EALLOW;
	IER |= M_INT3;
	PieCtrlRegs.PIEIER3.bit.INTx1=1;
   	EDIS;

	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global real time interrupt DBGM

	EPwm1Regs.CMPA.half.CMPA = 0;		   //Initialize the CMP register (Q2,Q3 / FB1,FB2)
#if FULL_BRIDGE==1
	EPwm2Regs.CMPA.half.CMPA = 0;		   //Initialize the CMP register (Q4,Q5 / FB3,FB4)
    mALL_PWM_ON(); //turn on the HEMTs that discharge the inductor
#else
    mP_PWM_ON(); //turn on the HEMT that discharges the inductor
#endif

	for(;;)  //infinite loop
	{
		EPwm1Regs.DBFED = DeadTime;
		EPwm1Regs.DBRED = DeadTime;
		EPwm2Regs.DBFED = DeadTime;
		EPwm2Regs.DBRED = DeadTime;
	} //END MAIN CODE
}

__interrupt void epwm1_timer_isr(void)
{
	if(Reset==1)
	{
		Reset=0;
		EPwm1TimerIntCount=0;
#if FULL_BRIDGE==1
		if(Polarity==0)
		{
			EPwm1Regs.CMPA.half.CMPA = CMP;		   //Initialize the CMP register (Q2,Q3 / FB1,FB2)
			EPwm2Regs.CMPA.half.CMPA = 0;		   //Initialize the CMP register (Q4,Q5 / FB3,FB4)
		}
		else
		{
			EPwm1Regs.CMPA.half.CMPA = 0;		   //Initialize the CMP register (Q2,Q3 / FB1,FB2)
			EPwm2Regs.CMPA.half.CMPA = CMP;		   //Initialize the CMP register (Q4,Q5 / FB3,FB4)
		}
#else
		EPwm1Regs.CMPA.half.CMPA = CMP;		   //Initialize the CMP register (Q2,Q3 / FB1,FB2)
#endif
	}
	if(EPwm1TimerIntCount<Pulses)
	{
	   EPwm1TimerIntCount++;
	}
	else
	{
		EPwm1Regs.CMPA.half.CMPA = 0;		   //Initialize the CMP register
#if FULL_BRIDGE==1
		EPwm2Regs.CMPA.half.CMPA = 0;		   //Initialize the CMP register
#endif
	}
   // Clear INT flag for this timer
   EPwm1Regs.ETCLR.bit.INT = 1;

   // Acknowledge this interrupt to receive more interrupts from group 3
   PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}

